@@ -12,7 +12,7 @@ use AnyEvent::Handle;
 use List::Util 'max';
 
 
-our $VERSION = '0.2';
+our $VERSION = '0.21';
 
 
 my $EOL = chr(10);
@@ -30,7 +30,7 @@ sub new {
     my $class = shift;
     my $self = $class->SUPER::new;
 
-    $self->{connection_timeout_margin} = 100;
+    $self->{connection_timeout_margin} = 250;
     $self->{connected} = 0;
     $self->{counter} = 0;
 
@@ -74,6 +74,8 @@ sub connect {
 
     croak "You already have established a connection." if $self->is_connected;
 
+    $self->{subscriptions} = {};
+
     $self->{handle} = AnyEvent::Handle->new(
         connect => [$self->{host}, $self->{port}],
         keep_alive => 1,
@@ -83,12 +85,12 @@ sub connect {
         on_connect_error => sub {
             shift->destroy;
             $self->{connected} = 0;
-            $self->event('CONNECTION_LOST', $self->{host}, $self->{port});
+            $self->event('CONNECT_ERROR', $self->{host}, $self->{port});
         },
         on_error => sub {
             shift->destroy;
+            $self->event('CONNECTION_LOST', $self->{host}, $self->{port}) if $self->{connected};
             $self->{connected} = 0;
-            $self->event('CONNECTION_LOST', $self->{host}, $self->{port});
         },
         on_read => sub {
             $self->read_frame;
@@ -197,8 +199,10 @@ sub reset_server_heartbeat_timer {
     $self->{heartbeat}{timer}{server} = AnyEvent->timer(
         after => (($interval+$self->get_connection_timeout_margin)/1000),
         cb => sub {
-            $self->{connected} = 0;
-            $self->event('CONNECTION_LOST', $self->{host}, $self->{port});
+            if ($self->{connected}) {
+                $self->{connected} = 0;
+                $self->event('CONNECTION_LOST', $self->{host}, $self->{port});
+            }
         }
     );
 }
@@ -564,6 +568,10 @@ sub on_connection_lost {
     return shift->reg_cb('CONNECTION_LOST', shift);
 }
 
+sub on_connect_error {
+    return shift->reg_cb('CONNECT_ERROR', shift);
+}
+
 sub on_send_frame {
     return shift->reg_cb('SEND_FRAME', shift);
 }
@@ -907,11 +915,17 @@ Parameters passed to the callback: C<$self>, C<$host>, C<$port>.
 
 =head3 $guard = $client->on_connection_lost $callback
 
-Invoked when either the C<on_error> or the C<on_connect_error> callback
-specified in the C<AnyEvent::Handle> constructor are called, or when no more
+Invoked when either the C<on_error> callback specified in the
+C<AnyEvent::Handle> constructor is called, or when no more
 heartbeats arrive from the server.
 Parameters passed to the callback: C<$self>, C<$host>, C<$port>.
 
+=head3 $guard = $client->on_connect_error $callback
+
+Invoked when the C<on_connect_error> callback specified in the
+C<AnyEvent::Handle> constructor is called.
+Parameters passed to the callback: C<$self>, C<$host>, C<$port>.
+
 =head3 $guard = $client->on_send_frame $callback
 
 Invoked when a STOMP frame is sent. Parameters passed to the callback:
